@@ -27,6 +27,7 @@ import apt_pkg
 import sys
 import os
 import subprocess
+import locale
 import logging
 import re
 import statvfs
@@ -36,10 +37,9 @@ import time
 import copy
 import ConfigParser
 from stat import *
-from utils import country_mirror, url_downloadable
+from utils import country_mirror, url_downloadable, check_and_fix_xbit, ExecutionTime
 from string import Template
 
-
 import DistUpgradeView
 from DistUpgradeConfigParser import DistUpgradeConfig
 from DistUpgradeFetcherCore import country_mirror
@@ -57,6 +57,7 @@ import gettext
 from DistUpgradeCache import *
 from DistUpgradeApport import *
 
+REBOOT_REQUIRED_FILE = "/var/run/reboot-required"
 
 class NoBackportsFoundException(Exception):
     pass
@@ -67,7 +68,7 @@ class DistUpgradeController(object):
     
     def __init__(self, distUpgradeView, options=None, datadir=None):
         # setup the paths
-        localedir = "/usr/share/locale/update-manager/"
+        localedir = "/usr/share/locale/"
         if datadir == None:
             datadir = os.getcwd()
             localedir = os.path.join(datadir,"mo")
@@ -130,6 +131,7 @@ class DistUpgradeController(object):
         os.environ["PYCENTRAL_FORCE_OVERWRITE"] = "1"
         os.environ["PATH"] = "%s:%s" % (os.getcwd()+"/imported",
                                         os.environ["PATH"])
+        check_and_fix_xbit("./imported/invoke-rc.d")
 
         # set max retries
         maxRetries = self.config.getint("Network","MaxRetries")
@@ -143,6 +145,10 @@ class DistUpgradeController(object):
         apt_pkg.Config.Set("Acquire::http::Timeout","20")
         apt_pkg.Config.Set("Acquire::ftp::Timeout","20")
 
+        # no list cleanup here otherwise a "cancel" in the upgrade
+        # will not restore the full state (lists will be missing)
+        apt_pkg.Config.Set("Apt::Get::List-Cleanup", "false")
+
         # forced obsoletes
         self.forced_obsoletes = self.config.getlist("Distro","ForcedObsoletes")
         # list of valid mirrors that we can add
@@ -150,6 +156,8 @@ class DistUpgradeController(object):
         # debugging
         #apt_pkg.Config.Set("DPkg::Options::","--debug=0077")
 
+        # apt cron job
+        self._aptCronJobPerms = 0755
 
     def openCache(self, lock=True):
         logging.debug("openCache()")
@@ -168,7 +176,10 @@ class DistUpgradeController(object):
         # run dpkg --configure -a
         except CacheExceptionDpkgInterrupted, e:
             logging.warning("dpkg interrupted, calling dpkg --configure -a")
-            self._view.getTerminal().call(["dpkg","--configure","-a"])
+            cmd = ["dpkg","--configure","-a"]
+            if os.environ.get("DEBIAN_FRONTEND") == "noninteractive":
+                cmd.append("--force-confold")
+            self._view.getTerminal().call(cmd)
             self.cache = MyCache(self.config,
                                  self._view,
                                  self.quirks,
@@ -183,7 +194,7 @@ class DistUpgradeController(object):
                                "application first."));
             sys.exit(1)
         self.cache.partialUpgrade = self._partialUpgrade
-        logging.debug("/openCache()")
+        logging.debug("/openCache(), new cache size %i" % len(self.cache))
 
     def _isRemoteLogin(self):
         " check if we are running form a remote login "
@@ -194,16 +205,9 @@ class DistUpgradeController(object):
         # sudo cleans out SSH_ environment
         out = subprocess.Popen(["who","-m","--ips"],stdout=subprocess.PIPE).communicate()[0]
         logging.debug("who -m --ips: '%s'" % out)
-        # the host is in ()
-        if not "(" in out:
-            return False
-        # if we have a () parse it
-        ip = out.strip().rsplit('(')[1]
-        ip = ip.strip(')')
-        # if we have a ip here and it does not start with a
-        # ":" we have a remote login
         # FIXME: what about IPv6 ?
-        if not ip.startswith(":"):
+        # do regexp search for a IP 
+        if re.search("(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})$", out):
             return True
         return False
 
@@ -226,14 +230,17 @@ class DistUpgradeController(object):
         """
         pidfile = os.path.join("/var/run/release-upgrader-sshd.pid")
         if (not os.path.exists(pidfile) and self._isRemoteLogin()):
-            # check if the frontend supports ssh ugprades (see lp: #322482)
+            # check if the frontend supports ssh upgrades (see lp: #322482)
             if not self._viewSupportsSSH():
                 logging.error("upgrade over ssh not alllowed")
                 self._view.error(_("Upgrading over remote connection not supported"),
                                  _("You are running the upgrade over a "
                                    "remote ssh connection with a frontend "
                                    "that does "
-                                   "not support this. The upgrade will "
+                                   "not support this. Please try a text "
+                                   "mode upgrade with 'do-release-upgrade'."
+                                   "\n\n"
+                                   "The upgrade will "
                                    "abort now. Please try without ssh.")
                                  )
                 sys.exit(1)
@@ -246,7 +253,7 @@ class DistUpgradeController(object):
                   "It is not recommended to perform a upgrade "
                   "over ssh currently because in case of failure "
                 "it is harder to recover.\n\n"
-                  "If you continue, a additional ssh daemon will be "
+                  "If you continue, an additional ssh daemon will be "
                   "started at port '%s'.\n"
                   "Do you want to continue?") % port)
             # abort
@@ -275,7 +282,8 @@ class DistUpgradeController(object):
         # check if we run from a LTS 
         forceLTS=False
         if (self.release == "dapper" or
-            self.release == "hardy"):
+            self.release == "hardy" or
+            self.release == "lucid"):
             forceLTS=True
         m = MetaReleaseCore(useDevelopmentRelease=False,
                             forceLTS=forceLTS)
@@ -511,7 +519,7 @@ class DistUpgradeController(object):
         foundToDist = False
         # collect information on what components (main,universe) are enabled for what distro (sub)version
         # e.g. found_components = { 'hardy':set("main","restricted"), 'hardy-updates':set("main") }
-        found_components = {}
+        self.found_components = {}
         for entry in self.sources.list[:]:
 
             # ignore invalid records or disabled ones
@@ -521,10 +529,22 @@ class DistUpgradeController(object):
             # we disable breezy cdrom sources to make sure that demoted
             # packages are removed
             if entry.uri.startswith("cdrom:") and entry.dist == self.fromDist:
+                logging.debug("disabled '%s' cdrom entry (dist == fromDist)" % entry)
                 entry.disabled = True
                 continue
-            # ignore cdrom sources otherwise
+            # check if there is actually a lists file for them available
+            # and disable them if not
             elif entry.uri.startswith("cdrom:"):
+                # 
+                listdir = apt_pkg.Config.FindDir("Dir::State::lists")
+                if not os.path.exists("%s/%s%s_%s_%s" % 
+                                      (listdir,
+                                       apt_pkg.URItoFileName(entry.uri),
+                                       "dists",
+                                       entry.dist,
+                                       "Release")):
+                    logging.warning("disabling cdrom source '%s' because it has no  Release file" % entry)
+                    entry.disabled = True
                 continue
 
             # special case for archive.canonical.com that needs to
@@ -574,8 +594,9 @@ class DistUpgradeController(object):
                 validTo = True
                 if (entry.disabled or
                     entry.type == "deb-src" or
-                    entry.uri.startswith("http://security.ubuntu.com") or
-                    entry.uri.startswith("http://archive.canonical.com")):
+                    "/security.ubuntu.com" in entry.uri or
+                    "%s-security" % self.fromDist in entry.dist or
+                    "/archive.canonical.com" in entry.uri):
                     validTo = False
                 if entry.dist in toDists:
                     # so the self.sources.list is already set to the new
@@ -610,14 +631,14 @@ class DistUpgradeController(object):
                           "%s-security" % self.toDist]:
                     # create entry if needed, ignore disabled
                     # entries and deb-src
-                    found_components.setdefault(d,set())
+                    self.found_components.setdefault(d,set())
                     if (not entry.disabled and entry.dist == d and
                         entry.type == "deb"):
                         for comp in entry.comps:
                             # only sync components we know about
                             if not comp in sync_components:
                                 continue
-                            found_components[d].add(comp)
+                            self.found_components[d].add(comp)
                     
             # disable anything that is not from a official mirror
             if not validMirror:
@@ -636,12 +657,13 @@ class DistUpgradeController(object):
                 entry.uri.startswith("cdrom:") or entry.dist == self.toDist):
                 continue
             # now check for "$dist-updates" and "$dist-security" and add any inconsistencies
-            if found_components.has_key(entry.dist):
-                # add the delta between "hardy" comps and "hardy-updates" comps once
-                logging.info("fixing components inconsistency from '%s'" % entry)
-                entry.comps.extend(list(found_components[self.toDist]-found_components[entry.dist]))
-                logging.info("to new entry '%s'" % entry)
-                del found_components[entry.dist]
+            if self.found_components.has_key(entry.dist):
+                component_diff = self.found_components[self.toDist]-self.found_components[entry.dist]
+                if component_diff:
+                    logging.info("fixing components inconsistency from '%s'" % entry)
+                    entry.comps.extend(list(component_diff))
+                    logging.info("to new entry '%s'" % entry)
+                    del self.found_components[entry.dist]
         return foundToDist
 
     def updateSourcesList(self):
@@ -652,7 +674,7 @@ class DistUpgradeController(object):
             res = self._view.askYesNoQuestion(_("No valid mirror found"),
                              _("While scanning your repository "
                                "information no mirror entry for "
-                               "the upgrade was found."
+                               "the upgrade was found. "
                                "This can happen if you run a internal "
                                "mirror or if the mirror information is "
                                "out of date.\n\n"
@@ -660,7 +682,7 @@ class DistUpgradeController(object):
                                "'sources.list' file anyway? If you choose "
                                "'Yes' here it will update all '%s' to '%s' "
                                "entries.\n"
-                               "If you select 'no' the update will cancel."
+                               "If you select 'No' the upgrade will cancel."
                                ) % (self.fromDist, self.toDist))
             if res:
                 # re-init the sources and try again
@@ -674,7 +696,7 @@ class DistUpgradeController(object):
                     secon = _("After scanning your 'sources.list' no "
                               "valid entry for '%s' was found.\n\n"
                               "Should default entries for '%s' be "
-                              "added? If you select 'No', the update "
+                              "added? If you select 'No', the upgrade "
                               "will cancel.") % (self.fromDist, self.toDist)
                     if not self._view.askYesNoQuestion(prim, secon):
                         self.abort()
@@ -747,8 +769,7 @@ class DistUpgradeController(object):
         # check if we have packages in ReqReinst state that are not
         # downloadable
         logging.debug("doPostInitialUpdate")
-        if not self._partialUpgrade:
-            self.quirks.run("PostInitialUpdate")
+        self.quirks.run("PostInitialUpdate")
         if len(self.cache.reqReinstallPkgs) > 0:
             logging.warning("packages in reqReinstall state, trying to fix")
             self.cache.fixReqReinst(self._view)
@@ -845,42 +866,56 @@ class DistUpgradeController(object):
 
 
     def askDistUpgrade(self):
-        # check what packages got demoted and ask the user
-        # if those shall be removed
-        demotions = set()
-        demotions_file = self.config.get("Distro","Demotions")
-        if os.path.exists(demotions_file):
-            map(lambda pkgname: demotions.add(pkgname.strip()),
-                filter(lambda line: not line.startswith("#"),
-                       open(demotions_file).readlines()))
-        self.installed_demotions = [pkg.name for pkg in self.cache if pkg.isInstalled and pkg.name in demotions]
-        if len(self.installed_demotions) > 0:
-	    self.installed_demotions.sort()
-            logging.debug("demoted: '%s'" % " ".join(self.installed_demotions))
-            self._view.showDemotions(_("Support for some applications ended"),
-                                   _("Canonical Ltd. no longer provides "
-                                     "support for the following software "
-                                     "packages. You can still get support "
-                                     "from the community.\n\n"
-                                     "If you have not enabled community "
-                                     "maintained software (universe), "
-                                     "these packages will be suggested for "
-                                     "removal at the end of the upgrade."),
-                                     self.installed_demotions)
-            self._view.updateStatus(_("Calculating the changes"))
-        # FIXME: integrate this into main upgrade dialog!?!
+        self._view.updateStatus(_("Calculating the changes"))
         if not self.cache.distUpgrade(self._view, self.serverMode, self._partialUpgrade):
             return False
 
         if self.serverMode:
             if not self.cache.installTasks(self.tasks):
                 return False
+
+        # check what packages got demoted, we do this after the upgrade
+        # calculation to skip packages that are marked for removal anyway
+        # FIXME: integrate this into main upgrade dialog!?!
+        self.installed_demotions = self.cache.get_installed_demoted_packages()
+        if len(self.installed_demotions) > 0:
+	    self.installed_demotions.sort()
+            logging.debug("demoted: '%s'" % " ".join(self.installed_demotions))
+            logging.debug("found: %s" % self.found_components)
+            text = _("Canonical Ltd. no longer provides "
+                     "support for the following software "
+                     "packages. You can still get support "
+                     "from the community.")
+            # if universe is not there, explain that we will ask for removal
+            # of the packages later
+            if not "universe" in self.found_components[self.toDist]:
+                text =  _("Canonical Ltd. no longer provides "
+                          "support for the following software "
+                          "packages. You can still get support "
+                          "from the community.\n\n"
+                          "These packages will be suggested for "
+                          "removal at the end of the upgrade because "
+                          "you have not enabled the community maintained "
+                          "software channel (universe).")
+            self._view.showDemotions(_("Support for some applications ended"),
+                                     text,
+                                     self.installed_demotions)
+        # flush UI
+        self._view.processEvents()
+
+        # show changes and confirm
         changes = self.cache.getChanges()
+        self._view.processEvents()
+
         # log the changes for debugging
         self._logChanges()
+        self._view.processEvents()
+
         # check if we have enough free space 
         if not self._checkFreeSpace():
             return False
+        self._view.processEvents()
+
         # ask the user if he wants to do the changes
         res = self._view.confirmChanges(_("Do you want to start the upgrade?"),
                                         changes,
@@ -888,9 +923,8 @@ class DistUpgradeController(object):
         return res
 
     def _disableAptCronJob(self):
-        self._aptCronJobPerms = 0755
         if os.path.exists("/etc/cron.daily/apt"):
-            self._aptCronJobPerms = os.stat("/etc/cron.daily/apt")[ST_MODE]
+            #self._aptCronJobPerms = os.stat("/etc/cron.daily/apt")[ST_MODE]
             logging.debug("disabling apt cron job (%s)" % oct(self._aptCronJobPerms))
             os.chmod("/etc/cron.daily/apt",0644)
     def _enableAptCronJob(self):
@@ -905,6 +939,15 @@ class DistUpgradeController(object):
         currentRetry = 0
         fprogress = self._view.getFetchProgress()
         iprogress = self._view.getInstallProgress(self.cache)
+        # start slideshow
+        url = self.config.getWithDefault("Distro","SlideshowUrl",None)
+        if url:
+            try:
+                lang = locale.getdefaultlocale()[0].split('_')[0]
+            except:
+                logging.exception("getdefaultlocale")
+                lang = "en"
+            self._view.getHtmlView().open("%s#locale=%s" % (url, lang))
         # retry the fetching in case of errors
         maxRetries = self.config.getint("Network","MaxRetries")
         # FIXME: we get errors like 
@@ -920,11 +963,16 @@ class DistUpgradeController(object):
         # POSSIBLE workaround: keep the list-dir locked so that 
         #          no apt-get update can run outside from the release
         #          upgrader 
+        user_canceled = False
         while currentRetry < maxRetries:
             try:
                 pm = apt_pkg.GetPackageManager(self.cache._depcache)
                 fetcher = apt_pkg.GetAcquire(fprogress)
                 res = self.cache._fetchArchives(fetcher, pm)
+            except apt.cache.FetchCancelledException, e:
+                logging.info("user canceled")
+                user_canceled = True
+                break
             except IOError, e:
                 # fetch failed, will be retried
                 logging.error("IOError in cache.commit(): '%s'. Retrying (currentTry: %s)" % (e,currentRetry))
@@ -933,13 +981,19 @@ class DistUpgradeController(object):
             return True
         
         # maximum fetch-retries reached without a successful commit
-        logging.error("giving up on fetching after maximum retries")
-        self._view.error(_("Could not download the upgrades"),
-                         _("The upgrade is now aborted. Please check your "
-                           "Internet connection or "
-                           "installation media and try again. All files "
-                           "downloaded so far are kept."),
-                           "%s" % e)
+        if user_canceled:
+            self._view.information(_("Upgrade canceled"),
+                                   _("The upgrade cancels now and the original "
+                                     "system state is restored. You can resume "
+                                     "the upgrade at a later time."))
+        else:
+            logging.error("giving up on fetching after maximum retries")
+            self._view.error(_("Could not download the upgrades"),
+                             _("The upgrade is now aborted. Please check your "
+                               "Internet connection or "
+                               "installation media and try again. All files "
+                               "downloaded so far are kept."),
+                             "%s" % e)
         # abort here because we want our sources.list back
         self._enableAptCronJob()
         self.abort()
@@ -951,7 +1005,7 @@ class DistUpgradeController(object):
         #  env["force_start"] = "1"
         #  subprocess.call(["/etc/init.d/apport","start"], env=env)
         # but hardy and intrepid do not have the force_start yet
-        if not os.path.exists(fname):
+        if not (os.path.exists(fname) and os.path.exists("etc-default-apport")):
             return
         # copy the jaunty version of the conf file in place
         # (this avoids a conffile prompt later)
@@ -969,7 +1023,8 @@ class DistUpgradeController(object):
         iprogress = self._view.getInstallProgress(self.cache)
         # retry the fetching in case of errors
         maxRetries = self.config.getint("Network","MaxRetries")
-        self.quirks.run("StartUpgrade")
+        if not self._partialUpgrade:
+            self.quirks.run("StartUpgrade")
         while currentRetry < maxRetries:
             try:
                 res = self.cache.commit(fprogress,iprogress)
@@ -978,21 +1033,25 @@ class DistUpgradeController(object):
                 # if its a ordering bug we can cleanly revert to
                 # the previous release, no packages have been installed
                 # yet (LP: #328655, #356781)
-                pre_configure_errors = [
-                  "E:Internal Error, Could not perform immediate configuration",
-                  "E:Couldn't configure pre-depend "]
-                for preconf_error in pre_configure_errors:
-                    if str(e).startswith(preconf_error):
-                        logging.debug("detected preconfigure error, restorting state")
-                        self._enableAptCronJob()
-                        # FIXME: strings are not good, but we are in string freeze
-                        # currently
-                        msg = _("Error during commit")
-                        msg += "\n'%s'\n" % str(e)
-                        msg += _("Restoring original system state")
-                        self._view.error(_("Could not install the upgrades"), msg)
-                        # abort() exits cleanly
-                        self.abort()
+                if os.path.exists("/var/run/update-manager-apt-exception"):
+                    e = open("/var/run/update-manager-apt-exception").read()
+                    # if its a ordering bug we can cleanly revert but we need to write
+                    # a marker for the parent process to know its this kind of error
+                    pre_configure_errors = [
+                        "E:Internal Error, Could not perform immediate configuration",
+                        "E:Couldn't configure pre-depend "]
+                    for preconf_error in pre_configure_errors:
+                        if str(e).startswith(preconf_error):
+                            logging.debug("detected preconfigure error, restorting state")
+                            self._enableAptCronJob()
+                            # FIXME: strings are not good, but we are in string freeze
+                            # currently
+                            msg = _("Error during commit")
+                            msg += "\n'%s'\n" % str(e)
+                            msg += _("Restoring original system state")
+                            self._view.error(_("Could not install the upgrades"), msg)
+                            # abort() exits cleanly
+                            self.abort()
                 
                 # invoke the frontend now and show a error message
                 msg = _("The upgrade is now aborted. Your system "
@@ -1006,7 +1065,10 @@ class DistUpgradeController(object):
                                  "%s" % e)
                 self._view.error(_("Could not install the upgrades"), msg)
                 # installing the packages failed, can't be retried
-                self._view.getTerminal().call(["dpkg","--configure","-a"])
+                cmd = ["dpkg","--configure","-a"]
+                if os.environ.get("DEBIAN_FRONTEND") == "noninteractive":
+                    cmd.append("--force-confold")
+                self._view.getTerminal().call(cmd)
                 self._enableAptCronJob()
                 return False
             except IOError, e:
@@ -1014,6 +1076,18 @@ class DistUpgradeController(object):
                 logging.error("IOError in cache.commit(): '%s'. Retrying (currentTry: %s)" % (e,currentRetry))
                 currentRetry += 1
                 continue
+            except OSError, e:
+                logging.exception("cache.commit()")
+                # deal gracefully with:
+                #  OSError: [Errno 12] Cannot allocate memory
+                if e.errno == 12:
+                    self._enableAptCronJob()
+                    msg = _("Error during commit")
+                    msg += "\n'%s'\n" % str(e)
+                    msg += _("Restoring original system state")
+                    self._view.error(_("Could not install the upgrades"), msg)
+                    # abort() exits cleanly
+                    self.abort()
             # no exception, so all was fine, we are done
             self._enableAptCronJob()
             return True
@@ -1034,8 +1108,7 @@ class DistUpgradeController(object):
         # run the quirks handler that does does like things adding
         # missing groups or similar work arounds, only do it on real
         # upgrades
-        if not self._partialUpgrade:
-            self.quirks.run("PostUpgrade")
+        self.quirks.run("PostUpgrade")
         # check out what packages are cruft now
         # use self.{foreign,obsolete}_pkgs here and see what changed
         now_obsolete = self.cache._getObsoletesPkgs()
@@ -1089,22 +1162,25 @@ class DistUpgradeController(object):
             remove_candidates = set()
         logging.debug("remove_candidates: '%s'" % remove_candidates)
         logging.debug("Start checking for obsolete pkgs")
-        for pkgname in remove_candidates:
+        progress = self._view.getOpCacheProgress()
+        for (i, pkgname) in enumerate(remove_candidates):
+            progress.update((i/float(len(remove_candidates)))*100.0)
             if pkgname not in self.foreign_pkgs:
                 self._view.processEvents()
                 if not self.cache.tryMarkObsoleteForRemoval(pkgname, remove_candidates, self.foreign_pkgs):
                     logging.debug("'%s' scheduled for remove but not safe to remove, skipping", pkgname)
         logging.debug("Finish checking for obsolete pkgs")
+        progress.done()
 
         # get changes
         changes = self.cache.getChanges()
-        logging.debug("The following packages are remove candidates: %s" % " ".join([pkg.name for pkg in changes]))
+        logging.debug("The following packages are marked for removal: %s" % " ".join([pkg.name for pkg in changes]))
         summary = _("Remove obsolete packages?")
         actions = [_("_Keep"), _("_Remove")]
         # FIXME Add an explanation about what obsolete packages are
         #explanation = _("")
-        if len(changes) > 0 and \
-               self._view.confirmChanges(summary, changes, 0, actions, False):
+        if (len(changes) > 0 and 
+            self._view.confirmChanges(summary, changes, 0, actions, False)):
             fprogress = self._view.getFetchProgress()
             iprogress = self._view.getInstallProgress(self.cache)
             try:
@@ -1117,8 +1193,7 @@ class DistUpgradeController(object):
                                    "information. "),
                                    "%s" % e)
         # run stuff after cleanup
-        if not self._partialUpgrade:
-            self.quirks.run("PostCleanup")
+        self.quirks.run("PostCleanup")
         # run the post upgrade scripts that can do fixup like xorg.conf
         # fixes etc - only do on real upgrades
         if not self._partialUpgrade:
@@ -1138,13 +1213,14 @@ class DistUpgradeController(object):
             logging.debug("Running PostInstallScript: '%s'" % script)
             try:
                 # work around kde tmpfile problem where it eats permissions
-                os.chmod(script, 0755)
+                check_and_fix_xbit(script)
                 self._view.getTerminal().call([script], hidden=True)
             except Exception, e:
                 logging.error("got error from PostInstallScript %s (%s)" % (script, e))
         
     def abort(self):
         """ abort the upgrade, cleanup (as much as possible) """
+        logging.debug("abort called")
         if hasattr(self, "sources"):
             self.sources.restoreBackup(self.sources_backup_ext)
         if hasattr(self, "aptcdrom"):
@@ -1243,9 +1319,18 @@ class DistUpgradeController(object):
 
     def isMirror(self, uri):
         " check if uri is a known mirror "
+        uri = uri.rstrip("/")
         for mirror in self.valid_mirrors:
+            mirror = mirror.rstrip("/")
             if is_mirror(mirror, uri):
                 return True
+            # deal with mirrors like
+            #    deb http://localhost:9977/security.ubuntu.com/ubuntu intrepid-security main restricted
+            # both apt-debtorrent and apt-cacher use this (LP: #365537)
+            mirror_host_part = mirror.split("//")[1]
+            if uri.endswith(mirror_host_part):
+                logging.debug("found apt-cacher/apt-torrent style uri %s" % uri)
+                return True
         return False
 
     def _getPreReqMirrorLines(self, dumb=False):
@@ -1419,27 +1504,9 @@ class DistUpgradeController(object):
         else:
             args.append("--without-network")
         # work around kde being clever and removing the x bit
-        if not ((S_IMODE(os.stat(sys.argv[0])[ST_MODE]) & S_IXUSR) == S_IXUSR):
-            os.chmod(sys.argv[0], 0755)
+        check_and_fix_xbit(sys.argv[0])
         os.execve(sys.argv[0],args, os.environ)
 
-    def preDoDistUpgrade(self):
-        " this runs right before apt calls out to dpkg "
-        # kill update-notifier now to suppress reboot required
-        if os.path.exists("/usr/bin/killall"):
-            subprocess.call(["killall","-q","update-notifier"])
-        # check theme, crux is known to fail badly when upgraded 
-        # from dapper
-        if (self.fromDist == "dapper" and 
-            "DISPLAY" in os.environ and "SUDO_USER" in os.environ):
-            out = subprocess.Popen(["sudo","-u", os.environ["SUDO_USER"],
-                                    "./theme-switch-helper.py", "-g"],
-                                    stdout=subprocess.PIPE).communicate()[0]
-            if "Crux" in out:
-                subprocess.call(["sudo","-u", os.environ["SUDO_USER"],
-                                    "./theme-switch-helper.py", "--defaults"])
-        return True
-
     # this is the core
     def fullUpgrade(self):
         # sanity check (check for ubuntu-desktop, brokenCache etc)
@@ -1512,15 +1579,22 @@ class DistUpgradeController(object):
         # then open the cache (again)
         self._view.updateStatus(_("Checking package manager"))
         self.openCache()
-        # now check if we still have some key packages after the update
-        # if not something went seriously wrong
+        # re-check server mode because we got new packages (it may happen
+        # that the system had no sources.list entries and therefore no
+        # desktop file information)
+        self.serverMode = self.cache.needServerMode()
+        # now check if we still have some key packages available/downloadable
+        # after the update - if not something went seriously wrong
+        # (this happend e.g. during the intrepid->jaunty upgrade for some
+        #  users when de.archive.ubuntu.com was overloaded)
         for pkg in self.config.getlist("Distro","BaseMetaPkgs"):
-            if not self.cache.has_key(pkg):
+            if (not self.cache.has_key(pkg) or
+                not self.cache.anyVersionDownloadable(self.cache[pkg])):
                 # FIXME: we could offer to add default source entries here,
                 #        but we need to be careful to not duplicate them
                 #        (i.e. the error here could be something else than
                 #        missing sources entries but network errors etc)
-                logging.error("No '%s' after sources.list rewrite+update" % pkg) 
+                logging.error("No '%s' available/downloadable after sources.list rewrite+update" % pkg) 
                 self._view.error(_("Invalid package information"),
                                  _("After your package information was "
                                    "updated the essential package '%s' can "
@@ -1544,7 +1618,6 @@ class DistUpgradeController(object):
             self.abort()
 
         # now do the upgrade
-        self.preDoDistUpgrade()
         self._view.setStep(DistUpgradeView.STEP_INSTALL)
         self._view.updateStatus(_("Upgrading"))
         if not self.doDistUpgrade():
@@ -1569,10 +1642,11 @@ class DistUpgradeController(object):
         if self._view.confirmRestart():
             p = subprocess.Popen("/sbin/reboot")
             sys.exit(0)
+        return True
         
     def run(self):
         self._view.processEvents()
-        self.fullUpgrade()
+        return self.fullUpgrade()
     
     def doPartialUpgrade(self):
         " partial upgrade mode, useful for repairing "
@@ -1605,8 +1679,14 @@ class DistUpgradeController(object):
                                      "were errors during the upgrade "
                                      "process."))
             return False
-        self._view.information(_("Upgrade complete"),
-                               _("The partial upgrade was completed."))
+
+        if os.path.exists(REBOOT_REQUIRED_FILE):
+            # we can not talk to session management here, we run as root
+            if self._view.confirmRestart():
+                p = subprocess.Popen("/sbin/reboot")
+        else:
+            self._view.information(_("Upgrade complete"),
+                                   _("The partial upgrade was completed."))
         return True
 
 
